/*
gcc -Wall -lhpdf -DHPDF_SHARED creapdf_protegido.c -o creapdf_protegido.exe
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <setjmp.h>
#include <hpdf.h>

jmp_buf env;

void error_handler(HPDF_STATUS   error_no, HPDF_STATUS   detail_no, void *user_data)
{
    printf ("ERROR: numero=%04X, detalle=%u\n", (HPDF_UINT)error_no, (HPDF_UINT)detail_no);
    longjmp(env, 1);
}

int main (int argc, char **argv)
{
    HPDF_Doc  pdf;
    HPDF_Font font;
    HPDF_Page page;
    HPDF_REAL tw;
    char fname[256];
    char text[100];
    char ownerpass[100];
    char userpass[100];

    if (argc != 5) {
	    fprintf(stderr, "uso: creapdf_protegido [nombre-pdf] [mensaje] [password1] [password2]\n");
	    exit(1);
    }

    snprintf(fname, sizeof(fname), "%s.pdf", argv[1]);
    snprintf(text, sizeof(text), "%s", argv[2]);
    snprintf(ownerpass, sizeof(ownerpass), "%s", argv[3]);
    snprintf(userpass, sizeof(userpass), "%s", argv[4]);

    printf("creando PDF %s con mensaje %s, passwd1 %s y passwd2 %s\n", fname, text, ownerpass, userpass);

    pdf = HPDF_New (error_handler, NULL);
    if (!pdf) {
        fprintf(stderr, "error: HPDF_New\n");
        exit(1);
    }

    if (setjmp(env)) {
        HPDF_Free (pdf);
        exit(1);
    }

    font = HPDF_GetFont(pdf, "Helvetica", NULL);

    page = HPDF_AddPage(pdf);

    HPDF_Page_SetSize(page, HPDF_PAGE_SIZE_B5, HPDF_PAGE_LANDSCAPE);

    HPDF_Page_BeginText(page);
    HPDF_Page_SetFontAndSize(page, font, 20);
    tw = HPDF_Page_TextWidth(page, text);
    HPDF_Page_MoveTextPos(page, (HPDF_Page_GetWidth (page) - tw) / 2, (HPDF_Page_GetHeight (page)  - 20) / 2);
    HPDF_Page_ShowText(page, text);
    HPDF_Page_EndText(page);

    HPDF_SetPassword(pdf, ownerpass, userpass);

    HPDF_SaveToFile(pdf, fname);

    HPDF_Free(pdf);

    return 0;
}

