#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_hpdf.h"
#include <hpdf.h>

static function_entry hpdf_functions[] = {
	PHP_FE(HPDF_New, NULL)
	PHP_FE(HPDF_GetFont, NULL)
	PHP_FE(HPDF_AddPage, NULL)
	PHP_FE(HPDF_SetPassword, NULL)
	PHP_FE(HPDF_SaveToFile, NULL)
	PHP_FE(HPDF_Free, NULL)
	PHP_FE(HPDF_Page_SetSize, NULL)
	PHP_FE(HPDF_Page_BeginText, NULL)
	PHP_FE(HPDF_Page_SetFontAndSize, NULL)
	PHP_FE(HPDF_Page_MoveTextPos, NULL)
	PHP_FE(HPDF_Page_ShowText, NULL)
	PHP_FE(HPDF_Page_EndText, NULL)
	PHP_FE(HPDF_Page_TextWidth, NULL)
	PHP_FE(HPDF_Page_GetWidth, NULL)
	PHP_FE(HPDF_Page_GetHeight, NULL)
	{ NULL, NULL, NULL }
};

zend_module_entry hpdf_module_entry = {
	STANDARD_MODULE_HEADER,
	PHP_HPDF_EXTNAME,
	hpdf_functions,
	PHP_MINIT(hpdf),
	NULL,
	NULL,
	NULL,
	NULL,
	PHP_HPDF_VERSION,
	STANDARD_MODULE_PROPERTIES
};

#ifdef COMPILE_DL_HPDF
ZEND_GET_MODULE(hpdf)
#endif

int le_hpdf_doc;
int le_hpdf_page;
int le_hpdf_font;

static void error_handler(HPDF_STATUS error, HPDF_STATUS detail, void *user_data)
{
	php_printf("ERROR: number=%04X, detail=%u\n", (HPDF_UINT)error, (HPDF_UINT)detail);
}

static void php_hpdf_doc_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	php_hpdf_doc_t *doc = (php_hpdf_doc_t *)rsrc->ptr;
	if (doc) {
		HPDF_Free(doc->handle);
		efree(doc);
	}
}

PHP_MINIT_FUNCTION(hpdf)
{
	le_hpdf_doc = zend_register_list_destructors_ex(php_hpdf_doc_dtor, NULL, PHP_HPDF_DOC_RES_NAME, module_number);
	le_hpdf_page = zend_register_list_destructors_ex(NULL, NULL, PHP_HPDF_PAGE_RES_NAME, module_number);
	le_hpdf_font = zend_register_list_destructors_ex(NULL, NULL, PHP_HPDF_FONT_RES_NAME, module_number);
	REGISTER_LONG_CONSTANT("HPDF_PAGE_SIZE_B5", HPDF_PAGE_SIZE_B5, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("HPDF_PAGE_LANDSCAPE", HPDF_PAGE_LANDSCAPE, CONST_PERSISTENT | CONST_CS);
}

PHP_FUNCTION(HPDF_New)
{
	php_hpdf_doc_t *doc = emalloc(sizeof(php_hpdf_doc_t));
	doc->handle = HPDF_New(error_handler, NULL);
	if (!doc->handle) {
		efree(doc);
		php_printf("error: HPDF_New\n");
		RETURN_FALSE;
	}
	ZEND_REGISTER_RESOURCE(return_value, doc, le_hpdf_doc);
}

PHP_FUNCTION(HPDF_GetFont)
{
	zval *zdoc;
	php_hpdf_doc_t *doc;
	php_hpdf_font_t *font;
	char *fontname;
	int fontsize;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, 
				"rs", &zdoc, &fontname, &fontsize)){
		php_printf("error: HPDF_GetFont\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(doc, php_hpdf_doc_t*, &zdoc, -1, PHP_HPDF_DOC_RES_NAME, le_hpdf_doc);
	font = emalloc(sizeof(php_hpdf_font_t));
	font->handle = HPDF_GetFont(doc->handle, fontname, NULL);
	if (!font->handle) {
		efree(font);
		RETURN_FALSE;
	}
	ZEND_REGISTER_RESOURCE(return_value, font, le_hpdf_font);
}

PHP_FUNCTION(HPDF_AddPage)
{
	zval* zdoc;
	php_hpdf_doc_t *doc;
	php_hpdf_page_t *page;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &zdoc)){
		php_printf("error: HPDF_GetFont\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(doc, php_hpdf_doc_t*, &zdoc, -1, PHP_HPDF_DOC_RES_NAME, le_hpdf_doc);
	page = emalloc(sizeof(php_hpdf_page_t));
	page->handle = HPDF_AddPage(doc->handle);
	if (!page->handle) {
		efree(page);
		RETURN_FALSE;
	}
	ZEND_REGISTER_RESOURCE(return_value, page, le_hpdf_page);
}

PHP_FUNCTION(HPDF_Page_SetSize)
{
	zval* zpage;
	php_hpdf_page_t *page;
	long arg1;
	long arg2;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rll", &zpage, &arg1, &arg2)){
		php_printf("error: HPDF_Page_SetSize\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(page, php_hpdf_page_t*, &zpage, -1, PHP_HPDF_PAGE_RES_NAME, le_hpdf_page);
	HPDF_Page_SetSize(page->handle, arg1, arg2);
	RETURN_TRUE;
}

PHP_FUNCTION(HPDF_Page_BeginText)
{
	zval* zpage;
	php_hpdf_page_t *page;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &zpage)){
		php_printf("error: HPDF_Page_BeginText\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(page, php_hpdf_page_t*, &zpage, -1, PHP_HPDF_PAGE_RES_NAME, le_hpdf_page);
	HPDF_Page_BeginText(page->handle);
	RETURN_TRUE;
}

PHP_FUNCTION(HPDF_Page_SetFontAndSize)
{
	zval* zpage;
	zval* zfont;
	php_hpdf_page_t *page;
	php_hpdf_font_t *font;
	long size;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rrl", 
				&zpage, &zfont, &size)){
		php_printf("error: HPDF_Page_SetFontAndSize\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(page, php_hpdf_page_t*, &zpage, -1, PHP_HPDF_PAGE_RES_NAME, le_hpdf_page);
	ZEND_FETCH_RESOURCE(font, php_hpdf_font_t*, &zfont, -1, PHP_HPDF_FONT_RES_NAME, le_hpdf_font);
	HPDF_Page_SetFontAndSize(page->handle, font->handle, size);
	RETURN_TRUE;
}

PHP_FUNCTION(HPDF_Page_TextWidth)
{
	zval* zpage;
	php_hpdf_page_t *page;
	char *text;
	int textlen;
	float ret;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", 
				&zpage, &text, &textlen)){
		php_printf("error: HPDF_Page_TextWidth\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(page, php_hpdf_page_t*, &zpage, -1, PHP_HPDF_PAGE_RES_NAME, le_hpdf_page);
	ret = HPDF_Page_TextWidth(page->handle, text);
	RETURN_DOUBLE(ret);
}

PHP_FUNCTION(HPDF_Page_GetWidth)
{
	zval* zpage;
	php_hpdf_page_t *page;
	long ret;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
				&zpage)){
		php_printf("error: HPDF_Page_GetWidth\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(page, php_hpdf_page_t*, &zpage, -1, PHP_HPDF_PAGE_RES_NAME, le_hpdf_page);
	ret = HPDF_Page_GetWidth(page->handle);
	RETURN_LONG(ret);
}

PHP_FUNCTION(HPDF_Page_GetHeight)
{
	zval* zpage;
	php_hpdf_page_t *page;
	long ret;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
				&zpage)){
		php_printf("error: HPDF_Page_GetHeight\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(page, php_hpdf_page_t*, &zpage, -1, PHP_HPDF_PAGE_RES_NAME, le_hpdf_page);
	ret = HPDF_Page_GetHeight(page->handle);
	RETURN_LONG(ret);
}


PHP_FUNCTION(HPDF_Page_MoveTextPos)
{
	zval* zpage;
	php_hpdf_page_t *page;
	double x;
	double y;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rdd", 
				&zpage, &x, &y)){
		php_printf("error: HPDF_Page_MoveTextPos\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(page, php_hpdf_page_t*, &zpage, -1, PHP_HPDF_PAGE_RES_NAME, le_hpdf_page);
	HPDF_Page_MoveTextPos(page->handle, x, y);
	RETURN_TRUE;
}

PHP_FUNCTION(HPDF_Page_ShowText)
{
	zval* zpage;
	php_hpdf_page_t *page;
	char *text;
	int textlen;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs", 
				&zpage, &text, &textlen)){
		php_printf("error: HPDF_Page_ShowText\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(page, php_hpdf_page_t*, &zpage, -1, PHP_HPDF_PAGE_RES_NAME, le_hpdf_page);
	HPDF_Page_ShowText(page->handle, text);
	RETURN_TRUE;
}

PHP_FUNCTION(HPDF_Page_EndText)
{
	zval* zpage;
	php_hpdf_page_t *page;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
				&zpage)){
		php_printf("error: HPDF_Page_EndText\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(page, php_hpdf_page_t*, &zpage, -1, PHP_HPDF_PAGE_RES_NAME, le_hpdf_page);
	HPDF_Page_EndText(page->handle);
	RETURN_TRUE;
}

PHP_FUNCTION(HPDF_SetPassword)
{
	zval *zdoc;
	php_hpdf_doc_t *doc;
	char *pass1;
	int pass1len;
	char *pass2;
	int pass2len;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, 
				"rss", &zdoc, &pass1, &pass1len,
				&pass2, &pass2len)){
		php_printf("error: HPDF_SetPassword\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(doc, php_hpdf_doc_t*, &zdoc, -1, PHP_HPDF_DOC_RES_NAME, le_hpdf_doc);
	HPDF_SetPassword(doc->handle, pass1, pass2);
	RETURN_TRUE;
}

PHP_FUNCTION(HPDF_SaveToFile)
{
	zval *zdoc;
	php_hpdf_doc_t *doc;
	char *file;
	int filelen;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, 
				"rs", &zdoc, &file, &filelen)){
		php_printf("error: HPDF_SaveToFile\n");
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(doc, php_hpdf_doc_t*, &zdoc, -1, PHP_HPDF_DOC_RES_NAME, le_hpdf_doc);
	HPDF_SaveToFile(doc->handle, file);
	RETURN_TRUE;
}

PHP_FUNCTION(HPDF_Free)
{
	zval* zdoc;
	php_hpdf_doc_t *doc;
	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", &zdoc)){
		RETURN_FALSE;
	}
	ZEND_FETCH_RESOURCE(doc, php_hpdf_doc_t*, &zdoc, -1, PHP_HPDF_DOC_RES_NAME, le_hpdf_doc);
	/* ZEND_FETCH_RESOURCE llama RETURN_FALSE si el resource no es del tipo requerido
	   por lo que en este punto ya sabemos que tenemos un resource tipo HPDF_Doc valido */
	zend_list_delete(Z_LVAL_P(zdoc));
	RETURN_TRUE;
}

